﻿/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

using System;
using System.Collections.Generic;
using CeboMsrNet;
using System.IO;

/**
 * CeboMsr - C# example - Streaming Input.
 *
 * Multiple inputs are read multiple times at 300 Hz
 * and printed to the console.
 */
namespace StreamingIExample
{
    class StreamingIExample
    {
        /// <summary>
        /// Read blocks of data from device.
        /// </summary>
        /// <param name="device">Device to use for test.</param>
        private void StreamingInputTestLC(Device device)
        {
            // Construct selection that contains inputs to read from.
            IInput[] inputs = new IInput[] {
                device.SingleEndedInputs[0],
                device.SingleEndedInputs[1],
                device.DifferentialInputs[1],
                device.DigitalPorts[0],
                device.DigitalPorts[1]
            };

            // Prepare device with this collection ...
            device.SetupInputFrame(inputs);

            // Start sampling ...
            device.StartBufferedDataAcquisition(300, 20 * 25, false);

            // Read 20 x 25 frames using blocked read,
            // this function returns after *all* (25) requested frames are collected.
            for (int i = 0; i < 20; ++i)
            {
                // Read 25 frames ...
                IList<InputFrame> frames = device.ReadBlocking(25);

                // Write out the 1st one.
                InputFrame inFrame = frames[0];
                Console.WriteLine(
                    "DigitalPort #0: " + inFrame.GetDigitalPort(0) + ", " +
                    "DigitalPort #1: " + inFrame.GetDigitalPort(1) + ", " +
                    "SingleEnded #0: " + inFrame.GetSingleEnded(0) + " V, " +
                    "SingleEnded #1: " + inFrame.GetSingleEnded(1) + " V, " +
                    "Differential #1: " + inFrame.GetDifferential(1) + " V");
            }

            // Stop the DAQ.
            device.StopDataAcquisition();
        }

        /// <summary>
        /// Read blocks of data from device.
        /// </summary>
        /// <param name="device">Device to use for test.</param>
        private void StreamingInputTestStick(Device device)
        {
            // Construct selection that contains inputs to read from.
            IInput[] inputs = new IInput[] {
                device.SingleEndedInputs[0],
                device.SingleEndedInputs[1],
                device.DigitalPorts[0]
            };

            // Prepare device with this collection ...
            device.SetupInputFrame(inputs);

            // Start sampling ...
            device.StartBufferedDataAcquisition(300, 20 * 25, false);

            // Read 20 x 25 frames using blocked read,
            // this function returns after *all* (25) requested frames are collected.
            for (int i = 0; i < 20; ++i)
            {
                // Read 25 frames ...
                IList<InputFrame> frames = device.ReadBlocking(25);

                // Write out the 1st one.
                InputFrame inFrame = frames[0];
                Console.WriteLine(
                    "DigitalPort #0: " + inFrame.GetDigitalPort(0) + ", " +
                    "SingleEnded #0: " + inFrame.GetSingleEnded(0) + " V, " +
                    "SingleEnded #1: " + inFrame.GetSingleEnded(1) + " V");
            }

            // Stop the DAQ.
            device.StopDataAcquisition();
        }

        /// <summary>
        /// The examples main method.
        /// </summary>
        private void RunExample()
        {
            try
            {
                // Search for devices ...
                IList<Device> devices = LibraryInterface.Enumerate(DeviceType.All);

                // If at least one has been found, use the first one ...
                if (devices.Count > 0)
                {
                    Device device = devices[0];

                    // Open device, nothing can be done without doing this.
                    device.Open();

                    // Do some streaming ...
                    if (device.DeviceType == DeviceType.CeboLC)
                        StreamingInputTestLC(device);
                    else if (device.DeviceType == DeviceType.CeboStick)
                        StreamingInputTestStick(device);
                    else
                        throw new InvalidOperationException("No supported device detected.");

                    // Finalize device usage, this free's up the device, so it can be used
                    // again, including other applications.
                    device.Close();
                }
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        static void Main(string[] args)
        {
            new StreamingIExample().RunExample();
        }
    }
}
